<?php

/**
 * @package   FL Image Sitemap Plugin
 * @author    Дмитрий Васюков http://fictionlabs.ru
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\Utilities\ArrayHelper;

class plgAjaxFLImageSitemap extends JPlugin
{
	// Construct

	public function __construct($subject, $config)
	{
		parent::__construct($subject, $config);
		$this->loadLanguage();
 
        $this->app 		= JFactory::getApplication();
		$this->input 	= $this->app->input;
		$this->db 		= JFactory::getDbo();
	}

	// Get Rss

	public function onAjaxFLImageSitemap()
	{
		if ($this->input->getString('code') != $this->params->get('channel_code')) {
			return;
		}

		// Set Encoding
		JFactory::getDocument()->setMimeEncoding('application/xml');
    	JFactory::getDocument()->setType('xml');

		// Check Cache
		if ($this->params->get('enable_cache', 0)) {
			// Get Request
			$component 	= $this->input->getString('component', 'all');
			$page 		= $this->input->getInt('page', 1);

			$cacheId 	= md5(serialize($component.'_'.$page));
			$cache 		= JFactory::getCache('plg_flimagesitemap', '');

			$cache->setCaching(true);
			$cache->setLifeTime($this->params->get('cache_time', JFactory::getConfig()->get('cachetime', 1140)));

			$html 	= $cache->get($cacheId, 'plg_flimagesitemap');

			if (!$html) {
				$html = $this->getRss();

				$cache->store($html, $cacheId, 'plg_flimagesitemap');
			}
		} else {
			$html = $this->getRss();
		}

		return $html;
	}

	// Get All Image Function

	public function getRss()
	{
		// Get Request
		$component 	= $this->input->getString('component', 'all');
		$page 		= $this->input->getInt('page', 1);

		// Get the path for the layout file
		$path = JPluginHelper::getLayoutPath('ajax', 'flimagesitemap');

		$items = array();

		// Include Content Items
		if ($this->params->get('enable_content') && !in_array($component, array('zoo', 'k2'))) {
			$itemsContent = $this->getComContentContent($page);
			$items = array_merge($items, $itemsContent);
		}

		// Include Zoo Items
		if ($this->params->get('enable_zoo') && !in_array($component, array('content', 'k2'))) {
			$itemsZoo = $this->getComZooContent($page);
			$items = array_merge($items, $itemsZoo);
		}

		// Include K2 Items
		if ($this->params->get('enable_k2') && !in_array($component, array('zoo', 'content'))) {
			$itemsK2 = $this->getComK2Content($page);
			$items = array_merge($items, $itemsK2);
		}

		// Render Template
		ob_start();
		include $path;
		$html = ob_get_clean();

		return $html;
	}

	// Get Items From Zoo Component

	public function getComZooContent($page = 1)
	{
		// make sure ZOO exist
		jimport('joomla.filesystem.file');
		if (!JFile::exists(JPATH_ADMINISTRATOR.'/components/com_zoo/config.php')
				|| !JComponentHelper::getComponent('com_zoo', true)->enabled) {
			return;
		}

		// load zoo config
		require_once (JPATH_ADMINISTRATOR.'/components/com_zoo/config.php');

		// check if Zoo > 2.4 is loaded
		if (!class_exists('App')) {
			return;
		}

		// get the ZOO App instance
		$zoo = App::getInstance('zoo');

		// register plugin path
		if ($path = $zoo->path->path('root:plugins/flimagesitemap/')) {
			$zoo->path->register($path, 'flimagesitemap');
		}

		$zooItems 	= array();
		$items 		= $this->getZooItems($zoo, $page);

		foreach ($items as $key => $item) {
			$imagesFromText 	= array();
			$renderer 			= $zoo->renderer->create('item');
			$date 	  			= new JDate($item->created);

			$renderer->addPath(array($zoo->path->path('component.site:'), dirname(__FILE__)));
			$renderer->checkPositions('item.default', $item);

			$imagesFromPosition = $this->getImagesFromElements($renderer->getPositionElements('images'));

			if ($this->params->get('enable_images_from_text', 0)) {
				$imagesFromText = $this->getImagesFromText($renderer->renderPosition('text'));
			}

			$images = array_merge($imagesFromPosition, $imagesFromText);

			if (!empty($images)) {
				$zooItems[] = array(
					'link' 			=> JRoute::_($zoo->route->item($item, false), false, 2),
					'images' 		=> $images
				);
			}
		}

		return $zooItems;
	}

	// Get Zoo Items

	public function getZooItems($zoo, $page)
	{
		$params 		= $this->params;
		$order 			= 'created DESC';
		$applications 	= $params->get('zoo_apps');
		$types 			= $params->get('zoo_types');
		$limit 			= $params->get('zoo_count', 500);
		$offset 		= ($page - 1)*$limit;
		$conditions     = array();

		if ($applications) {
			$conditions[] = "application_id IN (".implode(',', $applications).")";
		}

		if ($types) {
			$conditions[] = "type IN ('".implode('\',\'', $types)."')";
		}

		if (!empty($conditions)) {
			$conditions = implode(' AND ', $conditions);
		}

        $result = $zoo->table->item->all(compact('conditions', 'order', 'limit', 'offset'));

        return $result;
	}

	// Get Items From Content Component

	public function getComContentContent($page = 1)
	{
		JLoader::register('ContentHelperRoute', JPATH_SITE . '/components/com_content/helpers/route.php');

		JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_content/models', 'ContentModel');

		$contentItems = array();

		// Get an instance of the generic articles model
		$model = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));

		// Set application parameters in model
		$app       = JFactory::getApplication();
		$appParams = $app->getParams();
		$model->setState('params', $appParams);

		$limit = (int) $this->params->get('content_count', 5);

		// Set the filters based on the module params
		$model->setState('list.start', ($page - 1)*$limit);
		$model->setState('list.limit', $limit);
		$model->setState('filter.published', 1);

		// Access filter
		$access     = !JComponentHelper::getParams('com_content')->get('show_noauth');
		$authorised = JAccess::getAuthorisedViewLevels(JFactory::getUser()->get('id'));
		$model->setState('filter.access', $access);

		// Category filter
		$model->setState('filter.category_id', $this->params->get('content_catid', array()));

		// User filter
		$userId = JFactory::getUser()->get('id');

		switch ($this->params->get('content_user_id'))
		{
			case 'by_me' :
				$model->setState('filter.author_id', (int) $userId);
				break;
			case 'not_me' :
				$model->setState('filter.author_id', $userId);
				$model->setState('filter.author_id.include', false);
				break;

			case '0' :
				break;

			default:
				$model->setState('filter.author_id', (int) $this->params->get('content_user_id'));
				break;
		}

		// Filter by language
		$model->setState('filter.language', $app->getLanguageFilter());

		//  Featured switch
		switch ($this->params->get('content_show_featured'))
		{
			case '1' :
				$model->setState('filter.featured', 'only');
				break;
			case '0' :
				$model->setState('filter.featured', 'hide');
				break;
			default :
				$model->setState('filter.featured', 'show');
				break;
		}

		// Set ordering
		$ordering = 'a.created';
		$dir      = 'DESC';

		$model->setState('list.ordering', $ordering);
		$model->setState('list.direction', $dir);

		$items = $model->getItems();

		foreach ($items as $item)
		{	
			$date 		   = new JDate($item->created);
			$item->slug    = $item->id . ':' . $item->alias;
			$item->catslug = $item->catid . ':' . $item->category_alias;
			$item->link 	= JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catid, $item->language), false, 2);
			$imagesFromText = array();

			if ($this->params->get('enable_images_from_text', 0)) {
				$imagesFromText = $this->getImagesFromText($item->introtext.$item->fulltext);
			}

			$images = array_merge($this->getImagesFromJson($item->images), $imagesFromText);

			if (!empty($images)) {
				$contentItems[] = array(
					'link' 			=> $item->link,
					'images' 		=> $images
				);
			}
		}

		return $contentItems;
	}

	// Get Items From Content Component

	public function getComK2Content($page = 1)
	{
		require_once(JPATH_SITE.'/components/com_k2/helpers/route.php');
		require_once(JPATH_SITE.'/components/com_k2/helpers/utilities.php');

		$k2Items 		= array();
		$catFilter 		= $this->params->get('k2_cat_filter', 0);
		$catId 			= $this->params->get('k2_catid', 0);
		$getChildren 	= $this->params->get('k2_get_children', 0);
		$featured 		= $this->params->get('k2_show_featured');
		$limit 			= $this->params->get('k2_count', 500);
		$image 			= $this->params->get('k2_add_image', 0);
		$imageSize 		= $this->params->get('k2_image_size', 'large');

		$jnow 			= JFactory::getDate();
		$now 			= $jnow->toSql();
		$nullDate 		= $this->db->getNullDate();

		$query 			= "SELECT DISTINCT i.*,";
		$query 			.= "c.name AS categoryname,c.id AS categoryid, c.alias AS categoryalias, c.params AS categoryparams";
		$query 			.= " FROM #__k2_items as i RIGHT JOIN #__k2_categories c ON c.id = i.catid";
		$query 	 		.= " WHERE i.published = 1 AND i.trash = 0 AND c.published = 1";
		$query 			.= " AND ( i.publish_up = ".$this->db->Quote($nullDate)." OR i.publish_up <= ".$this->db->Quote($now)." )";
		$query 			.= " AND ( i.publish_down = ".$this->db->Quote($nullDate)." OR i.publish_down >= ".$this->db->Quote($now)." )";


		if ($catFilter) {

			if (!is_null($catId)) {

				if (is_array($catId)) {

					if ($getChildren) {
						$itemListModel = K2Model::getInstance('Itemlist', 'K2Model');
						$categories = $itemListModel->getCategoryTree($catId);
						$sql = @implode(',', $categories);
						$query .= " AND i.catid IN ({$sql})";
					} else {
						JArrayHelper::toInteger($cid);
						$query .= " AND i.catid IN(".implode(',', $catId).")";
					}
				} else {
					if ($getChildren) {
						$itemListModel = K2Model::getInstance('Itemlist', 'K2Model');
						$categories = $itemListModel->getCategoryTree($catId);
						$sql = @implode(',', $categories);
						$query .= " AND i.catid IN ({$sql})";
					} else {
						$query .= " AND i.catid=".(int)$catId;
					}
				}
			}
		}
			
		if ($featured == '0') {
			$query .= " AND i.featured != 1";
		}

		if ($featured == '1') {
			$query .= " AND i.featured = 1";
		}

		$this->db->setQuery($query, ($page - 1)*$limit, $limit);
		$items = $this->db->loadObjectList();

		if(!empty($items)) {
			foreach ($items as $item) {	
				$item->link 	= JRoute::_(K2HelperRoute::getItemRoute($item->id.':'.urlencode($item->alias), $item->catid.':'.urlencode($item->categoryalias)), false, 2);
				$item->image 	= $this->getK2Image($item, $imageSize);

				$imagesFromText = array();

				if ($this->params->get('enable_images_from_text', 0)) {
					$imagesFromText = $this->getImagesFromText($item->introtext.$item->fulltext);
				}

				$images = array_merge($item->image, $imagesFromText);

				if (!empty($images)) {
					$k2Items[] = array(
						'link' 			=> $item->link,
						'images' 		=> $images
					);
				}
			}
		}

		return $k2Items;;
	}

	// Get Images From Content

	public function getImagesFromJson($string)
	{
		$images 	= array();
		$json 		= json_decode($string);

		if (!empty($json->image_fulltext)) {
			$imageFull = array(
				'src' 	=> JURI::root().$json->image_fulltext,
				'alt' 	=> $this->clearText($json->image_fulltext_alt),
				'title' => $this->clearText($json->image_fulltext_caption),
			);

			$images[] = $imageFull;
		}

		if (!empty($json->image_intro)) {
			$imageIntro = array(
				'src' 	=> JURI::root().$json->image_intro,
				'alt' 	=> $this->clearText($json->image_intro_alt),
				'title' => $this->clearText($json->image_intro_caption),
			);

			$images[] = $imageIntro;
		}

		return $images; 
	}

	// Get Images From Content

	public function getImagesFromElements($elements = array())
	{
		$images 	= array();

		if (!empty($elements)) {

			foreach ($elements as $key => $element) {
				$title 		= $this->clearText($element->getItem()->name);
				$imageData	= $element->data();

                if ($element->config->type == 'image') {
                    $image = array(
						'src' 	=> $this->clearText(JURI::root() . str_replace('\\', '/', $imageData['file'])),
						'alt' 	=> $imageData['title'] ? $this->clearText($imageData['title']) : $title,
						'title' => $imageData['title'] ? $this->clearText($$imageData['title']) : $title,
					);

					$images[] = $image;
                }

                if ($element->config->type == 'jbimage' || $element->config->type == 'jbgalleryimage') {

                    foreach ((array)$imageData as $i => $row) {
                        if (isset($row['file']) && $row['file']) {

                            $image = array(
								'src' 	=> $this->clearText(JURI::root() . str_replace('\\', '/', $row['file'])),
								'alt' 	=> (isset($row['alt']) && !empty($row['alt'])) ? $this->clearText($row['alt']) : $title,
								'title' => (isset($row['title']) && !empty($row['title'])) ? $this->clearText($row['title']) : $title,
							);

							$images[] = $image;
                        }
                    }
                }

                if ($element->config->type == 'jbgallery') {
                	$path = JPATH_ROOT . '/' . trim($element->config->directory, '/'). '/' . trim($imageData['value'], '/');

                	$files = JFolder::files($path, '.', false, true, array('.svn', 'CVS', '.DS_Store'));
			        $files = array_filter(
			            $files, create_function('$file', 'return preg_match("#(\.bmp|\.gif|\.jpg|\.jpeg|\.png)$#i", $file);')
			        );

			        foreach ((array)$files as $i => $file) {
                        $image = array(
							'src' 	=> JURI::root().$this->getImageRelativeUrl($file),
							'alt' 	=> $title,
							'title' => $title,
						);

						$images[] = $image;
                    }
                }
			}
		}

		return $images; 
	}

	// Get Images From String

	public function getImagesFromText($text, $attr = 'src')
	{
		$images = array();

		$dom = new domDocument;
		$dom->encoding = "UTF-8";
		$dom->preserveWhiteSpace = false;
		libxml_use_internal_errors(true);

		$dom->loadHTML('<meta http-equiv="Content-Type" content="text/html; charset=utf-8">'.mb_convert_encoding($text, 'HTML-ENTITIES', 'UTF-8'));

		$imagesFromDom = $dom->getElementsByTagName('img');

		if (!empty($imagesFromDom)) {
			foreach($imagesFromDom as $img) {
		        $url 	= $this->getImageUrl($img->getAttribute($attr));   
		        $alt 	= $img->getAttribute('alt') ? $img->getAttribute('alt') : $img->getAttribute('title');   
		        $title 	= $img->getAttribute('title'); 

		        $image = array(
		        	'src'	=> $url,
		        	'alt' 	=> $this->clearText($alt),
		        	'title' => $this->clearText($title),
		        );

		        $images[] = $image;
		    }
		}

		return $images;
	}    

	// Check Image Url

	public function getImageUrl($url)
	{
		if (strpos($url, 'http://') === 0 || strpos($url, 'https://') === 0) { // Check External Url
        	return $url;
        } else {
        	return JUri::root().trim($url, '\/');
        }
	}

	// Clear Text

	public function clearText($text)
	{
        return trim(addslashes(htmlspecialchars(strip_tags($text))));
    }

    // Get Image From Server Url
	public function getImageRelativeUrl($path)
	{
		return ltrim(preg_replace('/^'.preg_quote(str_replace(DIRECTORY_SEPARATOR, '/', JPATH_ROOT), '/').'/i', '', str_replace(DIRECTORY_SEPARATOR, '/', $path)), '/');
	}

	// Get K2 Image Function

	public function getK2Image($item, $size = 'xlarge')
	{
		$result = array();
		$id 	= $item->id;

		switch ($size) {
			case 'xsmall':
				if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_XS.jpg')) {
					$result[0]['src'] = JURI::root().'/media/k2/items/cache/'.md5("Image".$id).'_XS.jpg';
				}
				break;

			case 'small':
				if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_S.jpg')) {
					$result[0]['src'] = JURI::root().'/media/k2/items/cache/'.md5("Image".$id).'_S.jpg';
				}
				break;

			case 'medium':
				if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_M.jpg')) {
					$result[0]['src'] = JURI::root().'/media/k2/items/cache/'.md5("Image".$id).'_M.jpg';
				}
				break;

			case 'large':
				if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_L.jpg')) {
					$result[0]['src'] = JURI::root().'/media/k2/items/cache/'.md5("Image".$id).'_L.jpg';
				}
				break;

			case 'xlarge':
				if (JFile::exists(JPATH_SITE.'/media/k2/items/cache/'.md5("Image".$id).'_XL.jpg')) {
					$result[0]['src'] = JURI::root().'media/k2/items/cache/'.md5("Image".$id).'_XL.jpg';
				}
				break;
		}

		if (!empty($result)) {
			$result[0]['alt'] 	= $item->title;
			$result[0]['title'] = $item->title;
		}

		return $result;
	}
}